﻿using System;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Logging;
using Robware.Projects.Code;
using Robware.Projects.Gogs;

namespace Robware.Api.Projects.Controllers {
	[ApiController]
	[Route("[controller]")]
	public class CodeController : ControllerBase {
		private readonly ILogger<CodeController> _logger;
		private readonly IGitApi _gitApi;

		public CodeController(ILogger<CodeController> logger, IGitApi gitApi) {
			_logger = logger;
			_gitApi = gitApi;
		}

		private async Task<ActionResult<TResult>> Do<TResult, TException>(Func<Task<TResult>> method) where TException : Exception {
			try {
				return await method();
			}
			catch (TException e) {
				_logger.Log(LogLevel.Error, e.Message);
				return NotFound();
			}
		}

		[HttpGet(nameof(Repositories))]
		public async Task<ActionResult<Repository[]>> Repositories(string user) => await Do<Repository[], UserNotFoundException>(async ()=> (await _gitApi.GetRepositories(user)).ToArray());

		[HttpGet(nameof(Branches))]
		public async Task<ActionResult<Branch[]>> Branches(string user, string repository) => await Do<Branch[], RepositoryNotFoundException>(async() => (await _gitApi.GetBranches(user, repository)).ToArray());

		[HttpGet(nameof(Commit))]
		public async Task<ActionResult<Commit>> Commit(string user, string repository, string hash) => await Do<Commit, CommitNotFoundException>(async () => await _gitApi.GetCommit(user, repository, hash));
	}
}
