﻿using System.Collections.Generic;
using System.Linq;
using System.Net.Http;
using System.Threading.Tasks;
using Newtonsoft.Json;
using Robware.Projects.Code;
using Robware.Projects.Gogs.Models;
using Robware.Projects.Gogs.States;

namespace Robware.Projects.Gogs {
	public class GogsApi : IGitApi {
		private readonly HttpClient _client;

		public GogsApi(HttpClient client) {
			_client = client;
		}

		private async Task<T> Get<T>(string url) {
			var response = await _client.GetStringAsync(url);
			return JsonConvert.DeserializeObject<T>(response);
		}

		public async Task<IEnumerable<Repository>> GetRepositories(string user) {
			try {
				return (await Get<IEnumerable<RepositoryState>>($"api/v1/users/{user}/repos")).Select(state => new GogsRepository(state));
			}
			catch (HttpRequestException e) {
				throw new UserNotFoundException(user, e);
			}
		}

		public async Task<IEnumerable<Branch>> GetBranches(string user, string repository) {
			try {
				return (await Get<IEnumerable<BranchState>>($"api/v1/repos/{user}/{repository}/branches")).Select(state => new GogsBranch(state));
			}
			catch (HttpRequestException e) {
				throw new RepositoryNotFoundException(user, repository, e);
			}
		}

		public async Task<Commit> GetCommit(string user, string repository, string hash) {
			try {
				return new GogsCommit(await Get<CommitDetailsState>($"api/v1/repos/{user}/{repository}/commits/{hash}"));
			}
			catch (HttpRequestException e) {
				throw new CommitNotFoundException(user, repository, hash, e);
			}
		}
	}
}